/* This file is part of KNemo
   Copyright (C) 2004 Percy Leonhardt <percy@eris23.de>

   KNemo is free software; you can redistribute it and/or modify
   it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of
   the License, or (at your option) any later version.

   KNemo is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#include <qfile.h>
#include <qdict.h>
#include <qlayout.h>
#include <qcstring.h>
#include <qlistbox.h>
#include <qcheckbox.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qstringlist.h>
#include <qdatastream.h>

#include <kglobal.h>
#include <kconfig.h>
#include <klocale.h>
#include <dcopclient.h>
#include <kiconloader.h>
#include <kfiledialog.h>
#include <kinputdialog.h>
#include <kapplication.h>
#include <knotifydialog.h>
#include <kgenericfactory.h>

#include "configdlg.h"
#include "configdialog.h"

typedef KGenericFactory<ConfigDialog, QWidget> KNemoFactory;
K_EXPORT_COMPONENT_FACTORY( kcm_knemo, KNemoFactory("kcm_knemo") )

ConfigDialog::ConfigDialog( QWidget *parent, const char *name, const QStringList& )
    : KCModule( KNemoFactory::instance(), parent, name ),
      mDlg( new ConfigDlg( this ) )
{
    KGlobal::locale()->insertCatalogue("kcm_knemo");
    setupToolTipArray();
    load();

    QVBoxLayout* top = new QVBoxLayout(this);
    mDlg->pushButtonNew->setPixmap( SmallIcon( "filenew" ) );
    mDlg->pushButtonDelete->setPixmap( SmallIcon( "editdelete" ) );
    mDlg->pushButtonAddToolTip->setPixmap( SmallIcon( "1rightarrow" ) );
    mDlg->pushButtonRemoveToolTip->setPixmap( SmallIcon( "1leftarrow" ) );

    mSettingsDict.setAutoDelete( true );
    setButtons( KCModule::Default | KCModule::Apply );

    connect( mDlg->pushButtonNew, SIGNAL( clicked() ),
             this, SLOT( buttonNewSelected() ) );
    connect( mDlg->pushButtonDelete, SIGNAL( clicked() ),
             this, SLOT( buttonDeleteSelected() ) );
    connect( mDlg->pushButtonAddToolTip, SIGNAL( clicked() ),
             this, SLOT( buttonAddToolTipSelected() ) );
    connect( mDlg->pushButtonRemoveToolTip, SIGNAL( clicked() ),
             this, SLOT( buttonRemoveToolTipSelected() ) );
    connect( mDlg->pushButtonNotifications, SIGNAL( clicked() ),
             this, SLOT( buttonNotificationsSelected() ) );
    connect( mDlg->lineEditAlias, SIGNAL( textChanged( const QString& ) ),
             this, SLOT( aliasChanged( const QString& ) ) );
    connect( mDlg->lineEditStartText, SIGNAL( textChanged( const QString& ) ),
             this, SLOT( startMenuTextChanged( const QString& ) ) );
    connect( mDlg->lineEditStartCommand, SIGNAL( textChanged( const QString& ) ),
             this, SLOT( startCommandChanged( const QString& ) ) );
    connect( mDlg->lineEditStopText, SIGNAL( textChanged( const QString& ) ),
             this, SLOT( stopMenuTextChanged( const QString& ) ) );
    connect( mDlg->lineEditStopCommand, SIGNAL( textChanged( const QString& ) ),
             this, SLOT( stopCommandChanged( const QString& ) ) );
    connect( mDlg->checkBoxNotConnected, SIGNAL( toggled( bool ) ),
             this, SLOT( checkBoxNotConnectedToggled ( bool ) ) );
    connect( mDlg->checkBoxNotExisting, SIGNAL( toggled( bool ) ),
             this, SLOT( checkBoxNotExistingToggled ( bool ) ) );
    connect( mDlg->checkBoxCustom, SIGNAL( toggled( bool ) ),
             this, SLOT( checkBoxCustomToggled ( bool ) ) );
    connect( mDlg->checkBoxStartRoot, SIGNAL( toggled( bool ) ),
             this, SLOT( checkBoxStartRootToggled ( bool ) ) );
    connect( mDlg->checkBoxStopRoot, SIGNAL( toggled( bool ) ),
             this, SLOT( checkBoxStopRootToggled ( bool ) ) );
    connect( mDlg->listBoxInterfaces, SIGNAL( highlighted( const QString& ) ),
             this, SLOT( interfaceSelected( const QString& ) ) );

    // In case the user opened the control center via the context menu
    // this call to the daemon will deliver the interface the menu
    // belongs so. This way we can preselect the appropriate entry in the list.
    QCString replyType;
    QByteArray replyData, arg;
    QString selectedInterface = QString::null;
    if ( kapp->dcopClient()->call( "kded", "knemod", "getSelectedInterface()", arg, replyType, replyData ) )
    {
        QDataStream reply( replyData,  IO_ReadOnly );
        reply >> selectedInterface;
    }

    if ( selectedInterface != QString::null )
    {
        // Try to preselect the interface.
        unsigned int i;
        for ( i = 0; i < mDlg->listBoxInterfaces->count(); i++ )
        {
            QListBoxItem* item = mDlg->listBoxInterfaces->item( i );
            if ( item->text() == selectedInterface )
            {
                // Found it.
                mDlg->listBoxInterfaces->setSelected( i, true );
                break;
            }
        }
        if ( i == mDlg->listBoxInterfaces->count() )
            // Not found. Select first entry in list.
            mDlg->listBoxInterfaces->setSelected( 0, true );
    }
    else
        // No interface from daemon. Select first entry in list.
        mDlg->listBoxInterfaces->setSelected( 0, true );

    top->add( mDlg );
}

ConfigDialog::~ConfigDialog()
{
    delete mDlg;
}

void ConfigDialog::load()
{
    mSettingsDict.clear();
    mDlg->listBoxInterfaces->clear();
    KConfig* config = new KConfig( "knemorc", true );

    config->setGroup( "General" );
    mToolTipContent = config->readEntry( "ToolTipContent", "2" ).toInt();
    QStrList list;
    int numEntries = config->readListEntry( "Interfaces", list );

    if ( numEntries == 0 )
        return;

    char* interface;
    for ( interface = list.first(); interface; interface = list.next() )
    {
        QString group( "Interface_" );
        group += interface;
        InterfaceSettings* settings = new InterfaceSettings();
        if ( config->hasGroup( group ) )
        {
            config->setGroup( group );
            settings->alias = config->readEntry( "Alias" );
            settings->startMenuText = config->readEntry( "StartMenuText" );
            settings->startCommand = config->readEntry( "StartCommand" );
            settings->stopMenuText = config->readEntry( "StopMenuText" );
            settings->stopCommand = config->readEntry( "StopCommand" );
            settings->hideWhenNotAvailable = config->readBoolEntry( "HideWhenNotAvailable" );
            settings->hideWhenNotExisting = config->readBoolEntry( "HideWhenNotExisting" );
            settings->customMenuEntries = config->readBoolEntry( "CustomMenuEntries" );
            settings->startRunAsRoot = config->readBoolEntry( "StartRunAsRoot" );
            settings->stopRunAsRoot = config->readBoolEntry( "StopRunAsRoot" );
        }
        mSettingsDict.insert( interface, settings );
        mDlg->listBoxInterfaces->insertItem( interface );
    }
    delete config;

    // These things need to be here so that 'Reset' from the control
    // center is handled correctly.
    setupToolTipTab();
}

void ConfigDialog::save()
{
    KConfig* config = new KConfig( "knemorc", false );

    QStringList list;
    QDictIterator<InterfaceSettings> it( mSettingsDict );
    for ( ; it.current(); ++it )
    {
        list.append( it.currentKey() );
        InterfaceSettings* settings = it.current();
        config->setGroup( "Interface_" + it.currentKey() );
        if ( settings->alias.isEmpty() )
            config->deleteEntry( "Alias" );
        else
            config->writeEntry( "Alias", settings->alias );
        config->writeEntry( "StartMenuText", settings->startMenuText );
        config->writeEntry( "StartCommand", settings->startCommand );
        config->writeEntry( "StopMenuText", settings->stopMenuText );
        config->writeEntry( "StopCommand", settings->stopCommand );
        config->writeEntry( "HideWhenNotAvailable", settings->hideWhenNotAvailable );
        config->writeEntry( "HideWhenNotExisting", settings->hideWhenNotExisting );
        config->writeEntry( "CustomMenuEntries", settings->customMenuEntries );
        config->writeEntry( "StartRunAsRoot", settings->startRunAsRoot );
        config->writeEntry( "StopRunAsRoot", settings->stopRunAsRoot );
    }

    config->setGroup( "General" );
    config->writeEntry( "ToolTipContent", mToolTipContent );
    config->writeEntry( "Interfaces", list );
    config->sync();
    delete config;
    kapp->dcopClient()->send( "kded", "knemod", "reparseConfiguration()", "" );
}

void ConfigDialog::defaults()
{
    QFile proc( "/proc/net/dev" );
    if ( proc.open( IO_ReadOnly ) )
    {
        mSettingsDict.clear();
        mDlg->listBoxInterfaces->clear();

        QString file =  proc.readAll();
        QStringList content = QStringList::split( "\n", file );
        if ( content.count() > 2 )
        {
            for ( unsigned int i = 2; i < content.count(); i++ )
            {
                QString interface = content[i].simplifyWhiteSpace();
                interface = interface.left( interface.find( ':' ) );
                if ( interface == "lo" )
                    continue;

                InterfaceSettings* settings = new InterfaceSettings();
                settings->hideWhenNotAvailable = false;
                settings->hideWhenNotExisting = false;
                settings->customMenuEntries = false;
                settings->startRunAsRoot = false;
                settings->stopRunAsRoot = false;
                mSettingsDict.insert( interface, settings );
                mDlg->listBoxInterfaces->insertItem( interface );
            }
            if ( mDlg->listBoxInterfaces->count() > 0 )
            {
                mDlg->listBoxInterfaces->setSelected( 0, true );
            }
            else
            {
                mDlg->lineEditAlias->setText( QString::null );
                mDlg->lineEditStartText->setText( QString::null );
                mDlg->lineEditStartCommand->setText( QString::null );
                mDlg->lineEditStopText->setText( QString::null );
                mDlg->lineEditStopCommand->setText( QString::null );
                mDlg->checkBoxNotConnected->setChecked( false );
                mDlg->checkBoxNotExisting->setChecked( false );
                mDlg->checkBoxCustom->setChecked( false );
                mDlg->checkBoxStartRoot->setChecked( false );
                mDlg->checkBoxStopRoot->setChecked( false );
            }
        }
        proc.close();
    }
    else
        changed( false );
}

void ConfigDialog::buttonNewSelected()
{
    bool ok = false;
    QString ifname = KInputDialog::getText( i18n( "Add new interface" ),
                                            i18n( "Please enter the name of the interface to be monitored.\nIt should be something like 'eth1', 'wlan2' or 'ppp0'." ),
                                            QString::null,
                                            &ok );

    if ( ok )
    {
        mDlg->listBoxInterfaces->insertItem( ifname );
        mSettingsDict.insert( ifname, new InterfaceSettings() );
        mDlg->listBoxInterfaces->setSelected( mDlg->listBoxInterfaces->count() - 1, true );
        changed( true );
    }
}

void ConfigDialog::buttonDeleteSelected()
{
    QListBoxItem* selected = mDlg->listBoxInterfaces->selectedItem();

    if ( selected == 0 )
        return;

    // TODO: find a better way than blocking signals
    mSettingsDict.remove( selected->text() );
    mDlg->lineEditAlias->blockSignals( true );
    mDlg->lineEditAlias->setText( QString::null );
    mDlg->lineEditAlias->blockSignals( false );
    mDlg->lineEditStartText->blockSignals( true );
    mDlg->lineEditStartText->setText( QString::null );
    mDlg->lineEditStartText->blockSignals( false );
    mDlg->lineEditStartCommand->blockSignals( true );
    mDlg->lineEditStartCommand->setText( QString::null );
    mDlg->lineEditStartCommand->blockSignals( false );
    mDlg->lineEditStopText->blockSignals( true );
    mDlg->lineEditStopText->setText( QString::null );
    mDlg->lineEditStopText->blockSignals( false );
    mDlg->lineEditStopCommand->blockSignals( true );
    mDlg->lineEditStopCommand->setText( QString::null );
    mDlg->lineEditStopCommand->blockSignals( false );
    mDlg->checkBoxNotConnected->blockSignals( true );
    mDlg->checkBoxNotConnected->setChecked( false );
    mDlg->checkBoxNotConnected->blockSignals( false );
    mDlg->checkBoxNotExisting->blockSignals( true );
    mDlg->checkBoxNotExisting->setChecked( false );
    mDlg->checkBoxNotExisting->blockSignals( false );
    mDlg->checkBoxCustom->blockSignals( true );
    mDlg->checkBoxCustom->setChecked( false );
    mDlg->checkBoxCustom->blockSignals( false );
    mDlg->checkBoxStartRoot->blockSignals( true );
    mDlg->checkBoxStartRoot->setChecked( false );
    mDlg->checkBoxStartRoot->blockSignals( false );
    mDlg->checkBoxStopRoot->blockSignals( true );
    mDlg->checkBoxStopRoot->setChecked( false );
    mDlg->checkBoxStopRoot->blockSignals( false );
    mDlg->listBoxInterfaces->removeItem( mDlg->listBoxInterfaces->index( selected ) );
    changed( true );
}

void ConfigDialog::buttonAddToolTipSelected()
{
    // Support extended selection
    for ( int k = mDlg->listBoxAvailable->count() - 1; k >= 0; k-- )
    {
        if ( !mDlg->listBoxAvailable->isSelected( k ) )
            continue;

        QListBoxItem* selected = mDlg->listBoxAvailable->item( k );

        if ( selected == 0 )
            continue;

        // Find the index of the selected item in the tooltip array.
        int itemIndex = 0;
        for ( int i = 0; mToolTips[i].first != QString::null; i++ )
        {
            if ( mToolTips[i].first == selected->text() )
            {
                itemIndex = i;
                break;
            }
        }

        // Find the first item in the display list which has a larger
        // index in the tooltip array. We have to insert the selected
        // item just before this one.
        int newIndex = -1;
        for ( unsigned int i = 0; i < mDlg->listBoxDisplay->count(); i++ )
        {
            // For every item in the display list find its index in
            // the tooltip array.
            int siblingIndex = 0;
            QListBoxItem* item = mDlg->listBoxDisplay->item( i );
            for ( int j = 0; mToolTips[j].first != QString::null; j++ )
            {
                if ( mToolTips[j].first == item->text() )
                {
                    siblingIndex = j;
                    break;
                }
            }

            // Check if the index of this item is larger than the index
            // of the selected item.
            if ( siblingIndex > itemIndex )
            {
                // Insert the selected item at position newIndex placing it
                // directly in front of the sibling.
                newIndex = i;
                break;
            }
        }

        mDlg->listBoxAvailable->setSelected( selected, false );
        mDlg->listBoxAvailable->takeItem( selected );
        mDlg->listBoxDisplay->insertItem( selected, newIndex );
        if ( mDlg->listBoxAvailable->count() == 0 )
            mDlg->pushButtonAddToolTip->setEnabled( false );
        if ( mDlg->listBoxDisplay->count() == 1 )
            mDlg->pushButtonRemoveToolTip->setEnabled( true );

        mToolTipContent += mToolTips[itemIndex].second;
        changed( true );
    }
}

void ConfigDialog::buttonRemoveToolTipSelected()
{
    // Support extended selection
    for ( int k = mDlg->listBoxDisplay->count() - 1; k >= 0; k-- )
    {
        if ( !mDlg->listBoxDisplay->isSelected( k ) )
            continue;

        QListBoxItem* selected = mDlg->listBoxDisplay->item( k );

        if ( selected == 0 )
            continue;

        // Find the index of the selected item in the tooltip array.
        int itemIndex = 0;
        for ( int i = 0; mToolTips[i].first != QString::null; i++ )
        {
            if ( mToolTips[i].first == selected->text() )
            {
                itemIndex = i;
                break;
            }
        }

        // Find the first item in the available list which has a larger
        // index in the tooltip array. We have to insert the selected
        // item just before this one.
        int newIndex = -1;
        for ( unsigned int i = 0; i < mDlg->listBoxAvailable->count(); i++ )
        {
            // For every item in the available list find its index in
            // the tooltip array.
            int siblingIndex = 0;
            QListBoxItem* item = mDlg->listBoxAvailable->item( i );
            for ( int j = 0; mToolTips[j].first != QString::null; j++ )
            {
                if ( mToolTips[j].first == item->text() )
                {
                    siblingIndex = j;
                    break;
                }
            }

            // Check if the index of this item is larger than the index
            // of the selected item.
            if ( siblingIndex > itemIndex )
            {
                // Insert the selected item at position newIndex placing it
                // directly in front of the sibling.
                newIndex = i;
                break;
            }
        }

        mDlg->listBoxDisplay->setSelected( selected, false );
        mDlg->listBoxDisplay->takeItem( selected );
        mDlg->listBoxAvailable->insertItem( selected, newIndex );
        if ( mDlg->listBoxDisplay->count() == 0 )
            mDlg->pushButtonRemoveToolTip->setEnabled( false );
        if ( mDlg->listBoxAvailable->count() == 1 )
            mDlg->pushButtonAddToolTip->setEnabled( true );

        mToolTipContent -= mToolTips[itemIndex].second;
        changed( true );
    }
}

void ConfigDialog::buttonNotificationsSelected()
{
    KNotifyDialog dialog( this );
    dialog.addApplicationEvents( "knemo" );
    dialog.exec();
}

void ConfigDialog::interfaceSelected( const QString& interface )
{
    InterfaceSettings* settings = mSettingsDict[interface];
    mDlg->lineEditAlias->setText( settings->alias );
    mDlg->lineEditStartText->setText( settings->startMenuText );
    mDlg->lineEditStartCommand->setText( settings->startCommand );
    mDlg->lineEditStopText->setText( settings->stopMenuText );
    mDlg->lineEditStopCommand->setText( settings->stopCommand );
    mDlg->checkBoxNotConnected->setChecked( settings->hideWhenNotAvailable );
    mDlg->checkBoxNotExisting->setChecked( settings->hideWhenNotExisting );
    mDlg->checkBoxCustom->setChecked( settings->customMenuEntries );
    mDlg->checkBoxStartRoot->setChecked( settings->startRunAsRoot );
    mDlg->checkBoxStopRoot->setChecked( settings->stopRunAsRoot );
}

void ConfigDialog::aliasChanged( const QString& text )
{
    QListBoxItem* selected = mDlg->listBoxInterfaces->selectedItem();

    if ( selected == 0 )
        return;

    InterfaceSettings* settings = mSettingsDict[selected->text()];
    settings->alias = text;
    changed( true );
}

void ConfigDialog::startMenuTextChanged( const QString& text )
{
    QListBoxItem* selected = mDlg->listBoxInterfaces->selectedItem();

    if ( selected == 0 )
        return;

    InterfaceSettings* settings = mSettingsDict[selected->text()];
    settings->startMenuText = text;
    changed( true );
}

void ConfigDialog::startCommandChanged( const QString& text )
{
    QListBoxItem* selected = mDlg->listBoxInterfaces->selectedItem();

    if ( selected == 0 )
        return;

    InterfaceSettings* settings = mSettingsDict[selected->text()];
    settings->startCommand = text;
    changed( true );
}

void ConfigDialog::stopMenuTextChanged( const QString& text )
{
    QListBoxItem* selected = mDlg->listBoxInterfaces->selectedItem();

    if ( selected == 0 )
        return;

    InterfaceSettings* settings = mSettingsDict[selected->text()];
    settings->stopMenuText = text;
    changed( true );
}

void ConfigDialog::stopCommandChanged( const QString& text )
{
    QListBoxItem* selected = mDlg->listBoxInterfaces->selectedItem();

    if ( selected == 0 )
        return;

    InterfaceSettings* settings = mSettingsDict[selected->text()];
    settings->stopCommand = text;
    changed( true );
}

void ConfigDialog::checkBoxNotConnectedToggled( bool on )
{
    QListBoxItem* selected = mDlg->listBoxInterfaces->selectedItem();

    if ( selected == 0 )
        return;

    InterfaceSettings* settings = mSettingsDict[selected->text()];
    settings->hideWhenNotAvailable = on;
    changed( true );
}

void ConfigDialog::checkBoxNotExistingToggled( bool on )
{
    QListBoxItem* selected = mDlg->listBoxInterfaces->selectedItem();

    if ( selected == 0 )
        return;

    InterfaceSettings* settings = mSettingsDict[selected->text()];
    settings->hideWhenNotExisting = on;
    changed( true );
}

void ConfigDialog::checkBoxCustomToggled( bool on )
{
    QListBoxItem* selected = mDlg->listBoxInterfaces->selectedItem();

    if ( selected == 0 )
        return;

    InterfaceSettings* settings = mSettingsDict[selected->text()];
    settings->customMenuEntries = on;
    changed( true );
}

void ConfigDialog::checkBoxStartRootToggled( bool on )
{
    QListBoxItem* selected = mDlg->listBoxInterfaces->selectedItem();

    if ( selected == 0 )
        return;

    InterfaceSettings* settings = mSettingsDict[selected->text()];
    settings->startRunAsRoot = on;
    changed( true );
}

void ConfigDialog::checkBoxStopRootToggled( bool on )
{
    QListBoxItem* selected = mDlg->listBoxInterfaces->selectedItem();

    if ( selected == 0 )
        return;

    InterfaceSettings* settings = mSettingsDict[selected->text()];
    settings->stopRunAsRoot = on;
    changed( true );
}

void ConfigDialog::setupToolTipTab()
{
    mDlg->listBoxDisplay->clear();
    mDlg->listBoxAvailable->clear();

    for ( int i = 0; mToolTips[i].first != QString::null; i++ )
    {
        if ( mToolTipContent & mToolTips[i].second )
            mDlg->listBoxDisplay->insertItem( mToolTips[i].first );
        else
            mDlg->listBoxAvailable->insertItem( mToolTips[i].first );
    }

    if ( mDlg->listBoxDisplay->count() > 0 )
    {
        mDlg->listBoxDisplay->setSelected( 0, true );
        mDlg->pushButtonRemoveToolTip->setEnabled( true );
    }
    else
        mDlg->pushButtonRemoveToolTip->setEnabled( false );

    if ( mDlg->listBoxAvailable->count() > 0 )
    {
        mDlg->listBoxAvailable->setSelected( 0, true );
        mDlg->pushButtonAddToolTip->setEnabled( true );
    }
    else
        mDlg->pushButtonAddToolTip->setEnabled( false );
}

void ConfigDialog::setupToolTipArray()
{
    // Cannot make this data static as the i18n macro doesn't seem
    // to work when called to early i.e. before setting the catalogue.
    mToolTips[0] = QPair<QString, int>( i18n( "Interface" ), INTERFACE );
    mToolTips[1] = QPair<QString, int>( i18n( "Alias" ), ALIAS );
    mToolTips[2] = QPair<QString, int>( i18n( "Status" ), STATUS );
    mToolTips[3] = QPair<QString, int>( i18n( "Uptime" ), UPTIME );
    mToolTips[4] = QPair<QString, int>( i18n( "IP-Address" ), IP_ADDRESS );
    mToolTips[5] = QPair<QString, int>( i18n( "Subnet Mask" ), SUBNET_MASK );
    mToolTips[6] = QPair<QString, int>( i18n( "HW-Address" ), HW_ADDRESS );
    mToolTips[7] = QPair<QString, int>( i18n( "PtP-Address" ), PTP_ADDRESS );
    mToolTips[8] = QPair<QString, int>( i18n( "Packets Received" ), RX_PACKETS );
    mToolTips[9] = QPair<QString, int>( i18n( "Packets Sent" ), TX_PACKETS );
    mToolTips[10] = QPair<QString, int>( i18n( "Bytes Received" ), RX_BYTES );
    mToolTips[11] = QPair<QString, int>( i18n( "Bytes Sent" ), TX_BYTES );
    mToolTips[12] = QPair<QString, int>( i18n( "ESSID" ), ESSID );
    mToolTips[13] = QPair<QString, int>( i18n( "Mode" ), MODE );
    mToolTips[14] = QPair<QString, int>( i18n( "Frequency" ), FREQUENCY );
    mToolTips[15] = QPair<QString, int>( i18n( "Bit Rate" ), BIT_RATE );
    mToolTips[16] = QPair<QString, int>( i18n( "Signal/Noise" ), SIGNAL_NOISE );
    mToolTips[17] = QPair<QString, int>( i18n( "Link Quality" ), LINK_QUALITY );
    mToolTips[18] = QPair<QString, int>();
}

